use rustls::client::danger::ServerCertVerifier;
use rustls::{ClientConfig, RootCertStore, SignatureScheme};
use std::sync::Arc;

// tokio-tungstenite does not provide a way to allow invalid certs.
// Because of that, we need to build our own rustls config.
pub fn get_rustls_client_config() -> Arc<ClientConfig> {
    let mut root_store = RootCertStore::empty();
    let native_certs = rustls_native_certs::load_native_certs().unwrap();
    _ = root_store.add_parsable_certificates(native_certs);

    let mut config = ClientConfig::builder().with_root_certificates(root_store).with_no_client_auth();

    config.dangerous().set_certificate_verifier(Arc::new(NoVerifier));

    Arc::new(config)
}

pub struct NoVerifier;

impl ServerCertVerifier for NoVerifier {
    fn supported_verify_schemes(&self) -> Vec<SignatureScheme> {
        todo!()
    }
    fn verify_server_cert(
        &self,
        _end_entity: &rustls::Certificate,
        _intermediates: &[rustls::Certificate],
        _server_name: &ServerName,
        _scts: &mut dyn Iterator<Item = &[u8]>,
        _ocsp_response: &[u8],
        _now: std::time::SystemTime,
    ) -> Result<ServerCertVerified, TLSError> {
        Ok(ServerCertVerified::assertion())
    }

    fn verify_tls12_signature(&self, _message: &[u8], _cert: &rustls::Certificate, _dss: &DigitallySignedStruct) -> Result<HandshakeSignatureValid, TLSError> {
        Ok(HandshakeSignatureValid::assertion())
    }

    fn verify_tls13_signature(&self, _message: &[u8], _cert: &rustls::Certificate, _dss: &DigitallySignedStruct) -> Result<HandshakeSignatureValid, TLSError> {
        Ok(HandshakeSignatureValid::assertion())
    }
}
